enifed('glimmer-benchmarks/index', ['exports', 'glimmer-benchmarks/lib/bench-init', 'glimmer-benchmarks/lib/stats', 'glimmer-benchmarks/lib/bench-suites'], function (exports, _glimmerBenchmarksLibBenchInit, _glimmerBenchmarksLibStats, _glimmerBenchmarksLibBenchSuites) {
  'use strict';

  var Benchmark = _glimmerBenchmarksLibBenchInit;
  exports.Benchmark = Benchmark;
  var Stats = _glimmerBenchmarksLibStats;
  exports.Stats = Stats;
  var Suites = _glimmerBenchmarksLibBenchSuites.default;
  exports.Suites = Suites;
});

enifed('glimmer-benchmarks/lib/bench-infra', ['exports', 'glimmer-benchmarks/lib/bench', 'glimmer-benchmarks/lib/stats'], function (exports, _glimmerBenchmarksLibBench, _glimmerBenchmarksLibStats) {
    'use strict';

    var TestReporter = (function (_BenchmarkReporter) {
        babelHelpers.inherits(TestReporter, _BenchmarkReporter);

        function TestReporter() {
            _BenchmarkReporter.call(this);
            this.pre = document.createElement('pre');
            this.cycle = document.createElement('pre');
            this.cycle.id = "cycle";
            this.pre.id = "benchmark-log";
            document.body.appendChild(this.cycle);
            document.body.appendChild(this.pre);
        }

        TestReporter.prototype.progress = function progress(count, elapsed, stats) {
            var error = stats.rme.toFixed(2);
            this.cycle.innerHTML = count + ' samples in ' + toHumanTime(elapsed) + ' (' + toHumanTime(stats.mean) + ' ± ' + error + '%)';
        };

        TestReporter.prototype.error = function error(_error) {
            this.logln(_error['stack']);
            this.cycle.innerHTML = "Errored";
        };

        TestReporter.prototype.complete = function complete(benchmark, event) {
            var rawStats = benchmark.stats;
            if (rawStats.sample.length === 0) return;
            var stats = new _glimmerBenchmarksLibStats.default({ bucket_precision: rawStats.moe });
            stats.push(rawStats.sample);
            this.cycle.innerHTML = "Idle";
            var unit = humanUnitFor(stats.median(), stats.percentile(95), stats.percentile(99), stats.moe());
            this.logln('Samples:    ' + rawStats.sample.length);
            this.logln('Median:     ' + toHumanTime(stats.median(), unit, 4));
            this.logln('95%:        ' + toHumanTime(stats.percentile(95), unit, 4));
            this.logln('99%:        ' + toHumanTime(stats.percentile(99), unit, 4));
            this.logln('Confidence: ±' + toHumanTime(stats.moe(), unit, 4));
            this.logln('\n');
            this.logln('Distribution:');
            var div = document.createElement('div');
            div.className = 'histogram';
            var distribution = stats.iqr().distribution();
            var maxHeight = Math.max.apply(Math, distribution.filter(function (d) {
                return d;
            }).map(function (d) {
                return d.count;
            }));
            var widthPercentage = 100 / distribution.length;
            distribution.forEach(function (d, i) {
                var bar = document.createElement('div');
                bar.style.width = widthPercentage + '%';
                bar.style.height = 95 * d.count / maxHeight + '%';
                bar.style.bottom = '0px';
                bar.style.left = i * widthPercentage + '%';
                bar.className = 'bar';
                bar.title = d.count;
                div.appendChild(bar);
                var label = document.createElement('span');
                label.style.width = '100px';
                label.style.height = '20px';
                label.style.bottom = '-20px';
                label.style.left = i * widthPercentage + '%';
                label.style.transform = 'translateX(-50px)';
                label.style.textAlign = 'center';
                label.style.lineHeight = '20px';
                label.style.fontSize = '10px';
                label.className = 'label';
                label.innerText = toHumanTime(d.range[0], unit, 2, false);
                div.appendChild(label);
                var label2 = label.cloneNode(false);
                label2.style.left = (i + 1) * widthPercentage + '%';
                label2.innerText = toHumanTime(d.range[1], unit, 2, false);
                div.appendChild(label2);
            });
            document.body.appendChild(div);
            this.logln('\n');
        };

        TestReporter.prototype.logln = function logln(message) {
            var child = document.createElement('pre');
            child.innerText = message;
            this.pre.appendChild(child);
        };

        return TestReporter;
    })(_glimmerBenchmarksLibBench.BenchmarkReporter);

    exports.TestReporter = TestReporter;

    var TimeUnit;
    (function (TimeUnit) {
        TimeUnit[TimeUnit["ns"] = 1000000000] = "ns";
        TimeUnit[TimeUnit["\u00B5s"] = 1000000] = "\u00B5s";
        TimeUnit[TimeUnit["ms"] = 1000] = "ms";
        TimeUnit[TimeUnit["s"] = 1] = "s";
    })(TimeUnit || (TimeUnit = {}));
    ;
    function humanUnitFor() {
        for (var _len = arguments.length, seconds = Array(_len), _key = 0; _key < _len; _key++) {
            seconds[_key] = arguments[_key];
        }

        var units = seconds.map(function (time) {
            if (time < 1 / 10000000) {
                return TimeUnit.ns;
            } else if (time < 1 / 10000) {
                return TimeUnit.µs;
            } else if (time < 1 / 10) {
                return TimeUnit.ms;
            } else {
                return TimeUnit.s;
            }
        });
        return Math.min.apply(Math, units);
    }
    function toHumanTime(seconds) {
        var base = arguments.length <= 1 || arguments[1] === undefined ? humanUnitFor(seconds) : arguments[1];
        var precision = arguments.length <= 2 || arguments[2] === undefined ? 2 : arguments[2];
        var displayUnit = arguments.length <= 3 || arguments[3] === undefined ? true : arguments[3];
        return (function () {
            var number = (seconds * base).toFixed(precision);
            var unit = displayUnit ? TimeUnit[base] : '';
            return '' + number + unit;
        })();
    }

    var TestBenchmarkEnvironment = (function (_BenchmarkEnvironment) {
        babelHelpers.inherits(TestBenchmarkEnvironment, _BenchmarkEnvironment);

        function TestBenchmarkEnvironment() {
            _BenchmarkEnvironment.apply(this, arguments);
        }

        TestBenchmarkEnvironment.prototype.elapsed = function elapsed(timestamp) {
            return Date.now() - timestamp;
        };

        return TestBenchmarkEnvironment;
    })(_glimmerBenchmarksLibBench.BenchmarkEnvironment);

    exports.TestBenchmarkEnvironment = TestBenchmarkEnvironment;
});

enifed('glimmer-benchmarks/lib/bench-init', ['exports', 'glimmer-benchmarks/lib/bench', 'glimmer-benchmarks/lib/bench-infra', 'glimmer-benchmarks/lib/bench-suites'], function (exports, _glimmerBenchmarksLibBench, _glimmerBenchmarksLibBenchInfra, _glimmerBenchmarksLibBenchSuites) {
    'use strict';

    exports.init = init;

    function init() {
        var options = getJsonFromUrl();
        if (!options['suite']) {
            document.writeln('<h1>Please select a benchmark suite</h1>');
            document.writeln('<ul>');
            Object.keys(_glimmerBenchmarksLibBenchSuites.default).forEach(function (id) {
                document.writeln(' <li><a href="?suite=' + id + '">' + id + '</a></li>');
            });
            document.writeln('</ul>');
            return;
        }
        /* tslint:disable:no-require-imports */
        var scenarios = require('glimmer-benchmarks').Suites[options['suite']];
        /* tslint:enable:no-require-imports */
        var suite = new _glimmerBenchmarksLibBench.BenchmarkSuite();
        var env = new _glimmerBenchmarksLibBenchInfra.TestBenchmarkEnvironment();
        var reporter = new _glimmerBenchmarksLibBenchInfra.TestReporter();
        scenarios.forEach(function (S) {
            return suite.add(new S(reporter, env));
        });
        var div = document.createElement('div');
        div.id = "buttons";
        Object.keys(suite.scenarios).forEach(function (name) {
            var button = document.createElement('button');
            button.onclick = function () {
                return suite.run(name);
            };
            button.innerText = name;
            div.appendChild(button);
        });
        document.body.insertBefore(div, document.body.firstChild);
    }

    function getJsonFromUrl(hashBased) {
        var query = undefined;
        if (hashBased) {
            var pos = location.href.indexOf("?");
            if (pos === -1) return [];
            query = location.href.substr(pos + 1);
        } else {
            query = location.search.substr(1);
        }
        var result = {};
        query.split("&").forEach(function (part) {
            if (!part) return;
            part = part.split("+").join(" "); // replace every + with space, regexp-free version
            var eq = part.indexOf("=");
            var key = eq > -1 ? part.substr(0, eq) : part;
            var val = eq > -1 ? decodeURIComponent(part.substr(eq + 1)) : "";
            var from = key.indexOf("[");
            if (from === -1) result[decodeURIComponent(key)] = val;else {
                var to = key.indexOf("]");
                var index = decodeURIComponent(key.substring(from + 1, to));
                key = decodeURIComponent(key.substring(0, from));
                if (!result[key]) result[key] = [];
                if (!index) result[key].push(val);else result[key][index] = val;
            }
        });
        return result;
    }
});

enifed('glimmer-benchmarks/lib/bench-suites', ['exports', 'glimmer-benchmarks/lib/benchmarks/if', 'glimmer-benchmarks/lib/benchmarks/baseline', 'glimmer-benchmarks/lib/benchmarks/partials', 'glimmer-benchmarks/lib/benchmarks/components'], function (exports, _glimmerBenchmarksLibBenchmarksIf, _glimmerBenchmarksLibBenchmarksBaseline, _glimmerBenchmarksLibBenchmarksPartials, _glimmerBenchmarksLibBenchmarksComponents) {
    'use strict';

    exports.default = {
        Baseline: _glimmerBenchmarksLibBenchmarksBaseline.default,
        If: _glimmerBenchmarksLibBenchmarksIf.default,
        Partials: _glimmerBenchmarksLibBenchmarksPartials.default,
        Components: _glimmerBenchmarksLibBenchmarksComponents.default
    };
});

enifed('glimmer-benchmarks/lib/bench', ['exports', 'glimmer-util', 'glimmer-test-helpers', 'glimmer-object-reference'], function (exports, _glimmerUtil, _glimmerTestHelpers, _glimmerObjectReference) {
    // Goals:
    //
    // 1. ability to create new benchmark examples with templates etc. (ideally
    //    in separate subclasses)
    // 2. unit test the examples to make sure they actually work before benchmarking
    // 3. clean up the UI of the report (post-MVP)
    'use strict';

    Benchmark.support.decompilation = false;

    var BenchmarkScenario = (function () {
        function BenchmarkScenario(reporter, env) {
            this.reporter = reporter;
            this.env = env;
        }

        BenchmarkScenario.prototype.start = function start() {};

        BenchmarkScenario.prototype.tick = function tick(target, event) {
            var stats = target.stats;
            var times = target.times;

            this.moe = target.stats.moe;
            var elapsed = this.env.elapsed(times.timeStamp);
            this.reporter.progress(stats.sample.length, elapsed / 1000, stats);
        };

        BenchmarkScenario.prototype.error = function error(_error) {
            this.reporter.error(_error);
        };

        BenchmarkScenario.prototype.complete = function complete(event) {
            this.reporter.complete(event.target, event);
        };

        return BenchmarkScenario;
    })();

    exports.BenchmarkScenario = BenchmarkScenario;

    var BenchmarkEnvironment = function BenchmarkEnvironment() {};

    exports.BenchmarkEnvironment = BenchmarkEnvironment;

    var BenchmarkReporter = function BenchmarkReporter() {};

    exports.BenchmarkReporter = BenchmarkReporter;

    var BenchmarkSuite = (function () {
        function BenchmarkSuite() {
            this.scenarios = _glimmerUtil.dict();
        }

        BenchmarkSuite.prototype.add = function add(scenario) {
            var bench = new Benchmark({
                minSamples: 200,
                name: scenario.name,
                fn: scenario.run.bind(scenario),
                onStart: scenario.start.bind(scenario),
                onCycle: function (event) {
                    if (event.target.aborted) return;
                    scenario.tick(event.target, event);
                },
                onError: function (error) {
                    scenario.error(error.message);
                },
                onComplete: scenario.complete.bind(scenario)
            });
            this.scenarios[scenario.name] = bench;
        };

        BenchmarkSuite.prototype.run = function run(name) {
            return this.scenarios[name].run({ async: true });
        };

        return BenchmarkSuite;
    })();

    exports.BenchmarkSuite = BenchmarkSuite;

    var TemplateBenchmarkScenario = (function (_BenchmarkScenario) {
        babelHelpers.inherits(TemplateBenchmarkScenario, _BenchmarkScenario);

        function TemplateBenchmarkScenario() {
            _BenchmarkScenario.apply(this, arguments);
            this.compiled = null;
            this.context = null;
            this.glimmerEnv = new _glimmerTestHelpers.TestEnvironment();
        }

        TemplateBenchmarkScenario.prototype.start = function start() {
            var _this = this;

            var glimmerEnv = this.glimmerEnv;

            this.compiled = _glimmerTestHelpers.compile(this.template(), { env: this.glimmerEnv });
            this.context = this.renderContext();
            try {
                this.test(function () {
                    var parent = glimmerEnv.getDOM().createElement('div');
                    var contextRef = new _glimmerObjectReference.UpdatableReference(_this.context);
                    glimmerEnv.begin();
                    _this.compiled.render(contextRef, parent, new _glimmerTestHelpers.TestDynamicScope());
                    glimmerEnv.commit();
                    return parent;
                });
            } catch (e) {
                // console.assert(e);
                this.error(e);
                throw e;
            }
        };

        TemplateBenchmarkScenario.prototype.run = function run() {
            var glimmerEnv = this.glimmerEnv;
            var compiled = this.compiled;

            var parent = glimmerEnv.getDOM().createElement('div');
            var context = new _glimmerObjectReference.UpdatableReference(this.context);
            glimmerEnv.begin();
            compiled.render(context, parent, new _glimmerTestHelpers.TestDynamicScope());
            glimmerEnv.commit();
        };

        return TemplateBenchmarkScenario;
    })(BenchmarkScenario);

    exports.TemplateBenchmarkScenario = TemplateBenchmarkScenario;
});

enifed("glimmer-benchmarks/lib/benchmarks/baseline", ["exports", "glimmer-benchmarks/lib/bench"], function (exports, _glimmerBenchmarksLibBench) {
    "use strict";

    var SingleTextNodeScenario = (function (_TemplateBenchmarkScenario) {
        babelHelpers.inherits(SingleTextNodeScenario, _TemplateBenchmarkScenario);

        function SingleTextNodeScenario() {
            _TemplateBenchmarkScenario.apply(this, arguments);
            this.name = "single text node";
            this.description = "A simple static text template";
        }

        SingleTextNodeScenario.prototype.template = function template() {
            return "hi";
        };

        SingleTextNodeScenario.prototype.renderContext = function renderContext() {
            return {};
        };

        SingleTextNodeScenario.prototype.test = function test(render) {
            var result = render().outerHTML;
            if (result !== '<div>hi</div>') {
                throw new Error("Invalid render: " + result);
            }
        };

        return SingleTextNodeScenario;
    })(_glimmerBenchmarksLibBench.TemplateBenchmarkScenario);

    exports.default = [SingleTextNodeScenario];
});

enifed("glimmer-benchmarks/lib/benchmarks/components", ["exports", "glimmer-benchmarks/lib/bench"], function (exports, _glimmerBenchmarksLibBench) {
    "use strict";

    var NestedEmberishCurleyTaglessComponentsScenario = (function (_TemplateBenchmarkScenario) {
        babelHelpers.inherits(NestedEmberishCurleyTaglessComponentsScenario, _TemplateBenchmarkScenario);

        function NestedEmberishCurleyTaglessComponentsScenario() {
            _TemplateBenchmarkScenario.apply(this, arguments);
            this.name = "nested emberish curley tagless component";
            this.description = "10 outer components, each with 10 inner components";
        }

        NestedEmberishCurleyTaglessComponentsScenario.prototype.start = function start() {
            this.glimmerEnv.registerStaticTaglessComponent("parent-component", null, "<h1>{{parent.name}}</h1>\n        <ul>\n          {{#each parent.children key=\"@index\" as |child|}}\n            {{child-component child=child parent=parent}}\n          {{/each}}\n        </ul>");
            this.glimmerEnv.registerStaticTaglessComponent("child-component", null, "<li>{{child.name}} (from {{parent.name}})</li>");
            _TemplateBenchmarkScenario.prototype.start.call(this);
        };

        NestedEmberishCurleyTaglessComponentsScenario.prototype.template = function template() {
            return "{{#each parents key=\"@index\" as |parent|}}\n              {{parent-component parent=parent}}\n            {{/each}}";
        };

        NestedEmberishCurleyTaglessComponentsScenario.prototype.renderContext = function renderContext() {
            var context = {
                parents: []
            };
            for (var i = 0; i < 10; i++) {
                var _parent = { name: "Parent " + i, children: [] };
                context.parents.push(_parent);
                for (var j = 0; j < 10; j++) {
                    _parent.children.push({ name: "Child " + j });
                }
            }
            return context;
        };

        NestedEmberishCurleyTaglessComponentsScenario.prototype.test = function test(render) {
            var element = render();
            var has10Parents = element.getElementsByTagName('h1').length === 10;
            var hasParentText = element.getElementsByTagName('h1')[0].textContent === 'Parent 0';
            var has100Children = element.getElementsByTagName('li').length === 100;
            var hasChildText = element.getElementsByTagName('li')[0].textContent === 'Child 0 (from Parent 0)';
            if (!has10Parents || !hasParentText || !has100Children || !hasChildText) {
                throw new Error("Invalid render: " + element.outerHTML);
            }
        };

        return NestedEmberishCurleyTaglessComponentsScenario;
    })(_glimmerBenchmarksLibBench.TemplateBenchmarkScenario);

    exports.default = [NestedEmberishCurleyTaglessComponentsScenario];
});

enifed('glimmer-benchmarks/lib/benchmarks/if', ['exports', 'glimmer-benchmarks/lib/bench'], function (exports, _glimmerBenchmarksLibBench) {
    'use strict';

    var IfScenario = (function (_TemplateBenchmarkScenario) {
        babelHelpers.inherits(IfScenario, _TemplateBenchmarkScenario);

        function IfScenario() {
            _TemplateBenchmarkScenario.apply(this, arguments);
        }

        IfScenario.prototype.start = function start() {
            this.glimmerEnv.registerHelper('inline-if', function (_ref) {
                var cond = _ref[0];
                var truthy = _ref[1];
                var falsy = _ref[2];

                return cond ? truthy : falsy;
            });
            _TemplateBenchmarkScenario.prototype.start.call(this);
        };

        IfScenario.prototype.renderContext = function renderContext() {
            return {
                truthy: true,
                falsy: false,
                first: "Kris",
                last: "Selden"
            };
        };

        IfScenario.prototype.test = function test(render) {
            var result = render().outerHTML;
            if (result !== '<div>Kris Selden</div>') {
                throw new Error('Invalid render: ' + result);
            }
        };

        return IfScenario;
    })(_glimmerBenchmarksLibBench.TemplateBenchmarkScenario);

    var InlineIfScenario = (function (_IfScenario) {
        babelHelpers.inherits(InlineIfScenario, _IfScenario);

        function InlineIfScenario() {
            _IfScenario.apply(this, arguments);
            this.name = 'inline if';
            this.description = 'two `inline-if` helpers';
        }

        InlineIfScenario.prototype.template = function template() {
            return '{{inline-if truthy first "empty"}} {{inline-if falsy "empty" last}}';
        };

        return InlineIfScenario;
    })(IfScenario);

    var BlockIfScenario = (function (_IfScenario2) {
        babelHelpers.inherits(BlockIfScenario, _IfScenario2);

        function BlockIfScenario() {
            _IfScenario2.apply(this, arguments);
            this.name = 'block if';
            this.description = 'two `{{#if}}` blocks';
        }

        BlockIfScenario.prototype.template = function template() {
            return '{{#if truthy}}{{first}}{{else}}empty{{/if}} {{#if falsy}}empty{{else}}{{last}}{{/if}}';
        };

        return BlockIfScenario;
    })(IfScenario);

    exports.default = [InlineIfScenario, BlockIfScenario];
});

enifed("glimmer-benchmarks/lib/benchmarks/partials", ["exports", "glimmer-benchmarks/lib/bench"], function (exports, _glimmerBenchmarksLibBench) {
    "use strict";

    var StaticPartialScenario = (function (_TemplateBenchmarkScenario) {
        babelHelpers.inherits(StaticPartialScenario, _TemplateBenchmarkScenario);

        function StaticPartialScenario() {
            _TemplateBenchmarkScenario.apply(this, arguments);
            this.name = "static partial";
            this.description = "`{{partial 'greeting'}}`";
        }

        StaticPartialScenario.prototype.start = function start() {
            this.glimmerEnv.registerPartial('greeting', "Hi {{person1.name}} from a partial");
            _TemplateBenchmarkScenario.prototype.start.call(this);
        };

        StaticPartialScenario.prototype.template = function template() {
            return "[{{partial 'greeting'}}]";
        };

        StaticPartialScenario.prototype.renderContext = function renderContext() {
            return {
                person1: { name: 'Alex Joyce' }
            };
        };

        StaticPartialScenario.prototype.test = function test(render) {
            var result = render().outerHTML;
            if (result !== '<div>[Hi Alex Joyce from a partial]</div>') {
                throw new Error("Invalid render: " + result);
            }
        };

        return StaticPartialScenario;
    })(_glimmerBenchmarksLibBench.TemplateBenchmarkScenario);

    var DynamicPartialScenario = (function (_TemplateBenchmarkScenario2) {
        babelHelpers.inherits(DynamicPartialScenario, _TemplateBenchmarkScenario2);

        function DynamicPartialScenario() {
            _TemplateBenchmarkScenario2.apply(this, arguments);
            this.name = "dynamic partial";
            this.description = "`{{partial partialName}}`";
        }

        DynamicPartialScenario.prototype.start = function start() {
            this.glimmerEnv.registerPartial('greeting', "Hi {{person1.name}} from a partial");
            _TemplateBenchmarkScenario2.prototype.start.call(this);
        };

        DynamicPartialScenario.prototype.template = function template() {
            return "[{{partial partialName}}]";
        };

        DynamicPartialScenario.prototype.renderContext = function renderContext() {
            return {
                partialName: 'greeting',
                person1: { name: 'Ben Joyce' }
            };
        };

        DynamicPartialScenario.prototype.test = function test(render) {
            var result = render().outerHTML;
            if (result !== '<div>[Hi Ben Joyce from a partial]</div>') {
                throw new Error("Invalid render: " + result);
            }
        };

        return DynamicPartialScenario;
    })(_glimmerBenchmarksLibBench.TemplateBenchmarkScenario);

    var BaselineScenario = (function (_TemplateBenchmarkScenario3) {
        babelHelpers.inherits(BaselineScenario, _TemplateBenchmarkScenario3);

        function BaselineScenario() {
            _TemplateBenchmarkScenario3.apply(this, arguments);
            this.name = "baseline";
            this.description = "a template with no partial";
        }

        BaselineScenario.prototype.template = function template() {
            return "[Hi {{person1.name}} from a template]";
        };

        BaselineScenario.prototype.renderContext = function renderContext() {
            return {
                person1: { name: 'Sarah Galvin' }
            };
        };

        BaselineScenario.prototype.test = function test(render) {
            var result = render().outerHTML;
            if (result !== '<div>[Hi Sarah Galvin from a template]</div>') {
                throw new Error("Invalid render: " + result);
            }
        };

        return BaselineScenario;
    })(_glimmerBenchmarksLibBench.TemplateBenchmarkScenario);

    exports.default = [StaticPartialScenario, DynamicPartialScenario, BaselineScenario];
});

enifed("glimmer-benchmarks/lib/stats", ["exports"], function (exports) {
    /*
    Note that if your data is too large, there _will_ be overflow.
    */
    "use strict";

    exports.default = Stats;
    function asc(a, b) {
        return a - b;
    }
    var config_params = {
        bucket_precision: function (o, s) {
            if (typeof s !== "number" || s <= 0) {
                throw new Error("bucket_precision must be a positive number");
            }
            o._config.bucket_precision = s;
            o.buckets = [];
        },
        buckets: function (o, b) {
            if (!Array.isArray(b) || b.length === 0) {
                throw new Error("buckets must be an array of bucket limits");
            }
            o._config.buckets = b;
            o.buckets = [];
        },
        bucket_extension_interval: function (o, s) {
            if (s === undefined) return;
            if (typeof s !== "number" || s <= 0) {
                throw new Error("bucket_extension_interval must be a positive number");
            }
            o._config.bucket_extension_interval = s;
        },
        store_data: function (o, s) {
            if (typeof s !== "boolean") {
                throw new Error("store_data must be a true or false");
            }
            o._config.store_data = s;
        },
        sampling: function (o, s) {
            if (typeof s !== "boolean") {
                throw new Error("sampling must be a true or false");
            }
            o._config.sampling = s;
        }
    };

    function Stats(c) {
        this._config = { store_data: true };
        if (c) {
            for (var k in config_params) {
                if (c.hasOwnProperty(k)) {
                    config_params[k](this, c[k]);
                }
            }
        }
        this.reset();
    }

    Stats.prototype = {
        reset: function () {
            if (this._config.store_data) this.data = [];
            this.length = 0;
            this.sum = 0;
            this.sum_of_squares = 0;
            this.sum_of_logs = 0;
            this.sum_of_square_of_logs = 0;
            this.zeroes = 0;
            this.max = this.min = null;
            this._reset_cache();
            return this;
        },
        _reset_cache: function () {
            this._stddev = null;
            if (this._config.store_data) this._data_sorted = null;
        },
        _find_bucket: function (a) {
            var b = 0,
                e = undefined,
                l = undefined;
            if (this._config.buckets) {
                l = this._config.buckets.length;
                if (this._config.bucket_extension_interval && a >= this._config.buckets[l - 1]) {
                    e = a - this._config.buckets[l - 1];
                    b = e / this._config.bucket_extension_interval + l;
                    if (this._config.buckets[b] === undefined) this._config.buckets[b] = this._config.buckets[l - 1] + (e / this._config.bucket_extension_interval + 1) * this._config.bucket_extension_interval;
                    if (this._config.buckets[b - 1] === undefined) this._config.buckets[b - 1] = this._config.buckets[l - 1] + e;
                }
                for (; b < l; b++) {
                    if (a < this._config.buckets[b]) {
                        break;
                    }
                }
            } else if (this._config.bucket_precision) {
                b = Math.floor(a / this._config.bucket_precision);
            }
            return b;
        },
        _add_cache: function (a) {
            var tuple = [1],
                i = undefined;
            if (a instanceof Array) {
                tuple = a;
                a = tuple.shift();
            }
            this.sum += a * tuple[0];
            this.sum_of_squares += a * a * tuple[0];
            if (a === 0) {
                this.zeroes++;
            } else {
                this.sum_of_logs += Math.log(a) * tuple[0];
                this.sum_of_square_of_logs += Math.pow(Math.log(a), 2) * tuple[0];
            }
            this.length += tuple[0];
            if (tuple[0] > 0) {
                if (this.max === null || this.max < a) this.max = a;
                if (this.min === null || this.min > a) this.min = a;
            }
            if (this.buckets) {
                var b = this._find_bucket(a);
                if (!this.buckets[b]) this.buckets[b] = [0];
                this.buckets[b][0] += tuple.shift();
                for (i = 0; i < tuple.length; i++) this.buckets[b][i + 1] = (this.buckets[b][i + 1] | 0) + (tuple[i] | 0);
            }
            this._reset_cache();
        },
        _del_cache: function (a) {
            var tuple = [1],
                i = undefined;
            if (a instanceof Array) {
                tuple = a;
                a = tuple.shift();
            }
            this.sum -= a * tuple[0];
            this.sum_of_squares -= a * a * tuple[0];
            if (a === 0) {
                this.zeroes--;
            } else {
                this.sum_of_logs -= Math.log(a) * tuple[0];
                this.sum_of_square_of_logs -= Math.pow(Math.log(a), 2) * tuple[0];
            }
            this.length -= tuple[0];
            if (this._config.store_data) {
                if (this.length === 0) {
                    this.max = this.min = null;
                }
                if (this.length === 1) {
                    this.max = this.min = this.data[0];
                } else if (tuple[0] > 0 && (this.max === a || this.min === a)) {
                    var _i = this.length - 1;
                    if (_i >= 0) {
                        this.max = this.min = this.data[_i--];
                        while (_i-- >= 0) {
                            if (this.max < this.data[_i]) this.max = this.data[_i];
                            if (this.min > this.data[_i]) this.min = this.data[_i];
                        }
                    }
                }
            }
            if (this.buckets) {
                var b = this._find_bucket(a);
                if (this.buckets[b]) {
                    this.buckets[b][0] -= tuple.shift();
                    if (this.buckets[b][0] === 0) delete this.buckets[b];else for (i = 0; i < tuple.length; i++) this.buckets[b][i + 1] = (this.buckets[b][i + 1] | 0) - (tuple[i] | 0);
                }
            }
            this._reset_cache();
        },
        push: function () {
            var i = undefined,
                a = undefined,
                args = Array.prototype.slice.call(arguments, 0);
            if (args.length && args[0] instanceof Array) args = args[0];
            for (i = 0; i < args.length; i++) {
                a = args[i];
                if (this._config.store_data) this.data.push(a);
                this._add_cache(a);
            }
            return this;
        },
        push_tuple: function (tuple) {
            if (!this.buckets) {
                throw new Error("push_tuple is only valid when using buckets");
            }
            this._add_cache(tuple);
        },
        pop: function () {
            if (this.length === 0 || this._config.store_data === false) return undefined;
            var a = this.data.pop();
            this._del_cache(a);
            return a;
        },
        remove_tuple: function (tuple) {
            if (!this.buckets) {
                throw new Error("remove_tuple is only valid when using buckets");
            }
            this._del_cache(tuple);
        },
        reset_tuples: function (tuple) {
            var b = undefined,
                l = undefined,
                t = undefined,
                ts = tuple.length;
            if (!this.buckets) {
                throw new Error("reset_tuple is only valid when using buckets");
            }
            for (b = 0, l = this.buckets.length; b < l; b++) {
                if (!this.buckets[b] || this.buckets[b].length <= 1) {
                    continue;
                }
                for (t = 0; t < ts; t++) {
                    if (typeof tuple[t] !== 'undefined') {
                        this.buckets[b][t] = tuple[t];
                    }
                }
            }
        },
        unshift: function () {
            var i = undefined,
                a = undefined,
                args = Array.prototype.slice.call(arguments, 0);
            if (args.length && args[0] instanceof Array) args = args[0];
            i = args.length;
            while (i--) {
                a = args[i];
                if (this._config.store_data) this.data.unshift(a);
                this._add_cache(a);
            }
            return this;
        },
        shift: function () {
            if (this.length === 0 || this._config.store_data === false) return undefined;
            var a = this.data.shift();
            this._del_cache(a);
            return a;
        },
        amean: function () {
            if (this.length === 0) return NaN;
            return this.sum / this.length;
        },
        gmean: function () {
            if (this.length === 0) return NaN;
            if (this.zeroes > 0) return NaN;
            return Math.exp(this.sum_of_logs / this.length);
        },
        stddev: function () {
            if (this.length === 0) return NaN;
            var n = this.length;
            if (this._config.sampling) n--;
            if (this._stddev === null) this._stddev = Math.sqrt((this.length * this.sum_of_squares - this.sum * this.sum) / (this.length * n));
            return this._stddev;
        },
        gstddev: function () {
            if (this.length === 0) return NaN;
            if (this.zeroes > 0) return NaN;
            var n = this.length;
            if (this._config.sampling) n--;
            return Math.exp(Math.sqrt((this.length * this.sum_of_square_of_logs - this.sum_of_logs * this.sum_of_logs) / (this.length * n)));
        },
        moe: function () {
            if (this.length === 0) return NaN;
            // see http://en.wikipedia.org/wiki/Standard_error_%28statistics%29
            return 1.96 * this.stddev() / Math.sqrt(this.length);
        },
        range: function () {
            if (this.length === 0) return [NaN, NaN];
            return [this.min, this.max];
        },
        distribution: function () {
            if (this.length === 0) return [];
            if (!this.buckets) throw new Error("bucket_precision or buckets not configured.");
            var d = [],
                i = undefined,
                j = undefined,
                l = undefined;
            if (this._config.buckets) {
                j = this.min;
                l = Math.min(this.buckets.length, this._config.buckets.length);
                for (i = 0; i < l; j = this._config.buckets[i++]) {
                    if (this._config.buckets[i] === undefined && this._config.bucket_extension_interval) this._config.buckets[i] = this._config.buckets[i - 1] + this._config.bucket_extension_interval;
                    if (this.min > this._config.buckets[i]) continue;
                    d[i] = {
                        bucket: (j + this._config.buckets[i]) / 2,
                        range: [j, this._config.buckets[i]],
                        count: this.buckets[i] ? this.buckets[i][0] : 0,
                        tuple: this.buckets[i] ? this.buckets[i].slice(1) : []
                    };
                    if (this.max < this._config.buckets[i]) break;
                }
                if (i === l && this.buckets[i]) {
                    d[i] = {
                        bucket: (j + this.max) / 2,
                        range: [j, this.max],
                        count: this.buckets[i][0],
                        tuple: this.buckets[i] ? this.buckets[i].slice(1) : []
                    };
                }
            } else if (this._config.bucket_precision) {
                i = Math.floor(this.min / this._config.bucket_precision);
                l = Math.floor(this.max / this._config.bucket_precision) + 1;
                for (j = 0; i < l && i < this.buckets.length; i++, j++) {
                    d[j] = {
                        bucket: (i + 0.5) * this._config.bucket_precision,
                        range: [i * this._config.bucket_precision, (i + 1) * this._config.bucket_precision],
                        count: this.buckets[i] ? this.buckets[i][0] : 0,
                        tuple: this.buckets[i] ? this.buckets[i].slice(1) : []
                    };
                }
            }
            return d;
        },
        percentile: function (p) {
            if (this.length === 0 || !this._config.store_data && !this.buckets) return NaN;
            // If we come here, we either have sorted data or sorted buckets
            var v = undefined;
            if (p <= 0) v = 0;else if (p === 25) v = [Math.floor((this.length - 1) * 0.25), Math.ceil((this.length - 1) * 0.25)];else if (p === 50) v = [Math.floor((this.length - 1) * 0.5), Math.ceil((this.length - 1) * 0.5)];else if (p === 75) v = [Math.floor((this.length - 1) * 0.75), Math.ceil((this.length - 1) * 0.75)];else if (p >= 100) v = this.length - 1;else v = Math.floor(this.length * p / 100);
            if (v === 0) return this.min;
            if (v === this.length - 1) return this.max;
            if (this._config.store_data) {
                if (this._data_sorted === null) this._data_sorted = this.data.slice(0).sort(asc);
                if (typeof v === 'number') return this._data_sorted[v];else return (this._data_sorted[v[0]] + this._data_sorted[v[1]]) / 2;
            } else {
                var j = undefined;
                if (typeof v !== 'number') v = (v[0] + v[1]) / 2;
                if (this._config.buckets) j = 0;else if (this._config.bucket_precision) j = Math.floor(this.min / this._config.bucket_precision);
                for (; j < this.buckets.length; j++) {
                    if (!this.buckets[j]) continue;
                    if (v <= this.buckets[j][0]) {
                        break;
                    }
                    v -= this.buckets[j][0];
                }
                return this._get_nth_in_bucket(v, j);
            }
        },
        _get_nth_in_bucket: function (n, b) {
            var range = [];
            if (this._config.buckets) {
                range[0] = b > 0 ? this._config.buckets[b - 1] : this.min;
                range[1] = b < this._config.buckets.length ? this._config.buckets[b] : this.max;
            } else if (this._config.bucket_precision) {
                range[0] = Math.max(b * this._config.bucket_precision, this.min);
                range[1] = Math.min((b + 1) * this._config.bucket_precision, this.max);
            }
            return range[0] + (range[1] - range[0]) * n / this.buckets[b][0];
        },
        median: function () {
            return this.percentile(50);
        },
        iqr: function () {
            var q1 = undefined,
                q3 = undefined,
                fw = undefined;
            q1 = this.percentile(25);
            q3 = this.percentile(75);
            fw = (q3 - q1) * 1.5;
            return this.band_pass(q1 - fw, q3 + fw, true);
        },
        band_pass: function (low, high, open, config) {
            var i = undefined,
                j = undefined,
                b = undefined,
                b_val = undefined,
                i_val = undefined;
            if (!config) config = this._config;
            b = new Stats(config);
            if (this.length === 0) return b;
            if (this._config.store_data) {
                if (this._data_sorted === null) this._data_sorted = this.data.slice(0).sort(asc);
                for (i = 0; i < this.length && (this._data_sorted[i] < high || !open && this._data_sorted[i] === high); i++) {
                    if (this._data_sorted[i] > low || !open && this._data_sorted[i] === low) {
                        b.push(this._data_sorted[i]);
                    }
                }
            } else if (this._config.buckets) {
                for (i = 0; i <= this._config.buckets.length; i++) {
                    if (this._config.buckets[i] < this.min) continue;
                    b_val = i === 0 ? this.min : this._config.buckets[i - 1];
                    if (b_val < this.min) b_val = this.min;
                    if (b_val > this.max) b_val = this.max;
                    if (high < b_val || open && high === b_val) {
                        break;
                    }
                    if (low < b_val || !open && low === b_val) {
                        for (j = 0; j < (this.buckets[i] ? this.buckets[i][0] : 0); j++) {
                            i_val = Stats.prototype._get_nth_in_bucket.call(this, j, i);
                            if ((i_val > low || !open && i_val === low) && (i_val < high || !open && i_val === high)) {
                                b.push(i_val);
                            }
                        }
                    }
                }
                b.min = Math.max(low, b.min);
                b.max = Math.min(high, b.max);
            } else if (this._config.bucket_precision) {
                var low_i = Math.floor(low / this._config.bucket_precision),
                    high_i = Math.floor(high / this._config.bucket_precision) + 1;
                for (i = low_i; i < Math.min(this.buckets.length, high_i); i++) {
                    for (j = 0; j < (this.buckets[i] ? this.buckets[i][0] : 0); j++) {
                        i_val = Stats.prototype._get_nth_in_bucket.call(this, j, i);
                        if ((i_val > low || !open && i_val === low) && (i_val < high || !open && i_val === high)) {
                            b.push(i_val);
                        }
                    }
                }
                b.min = Math.max(low, b.min);
                b.max = Math.min(high, b.max);
            }
            return b;
        },
        copy: function (config) {
            var b = Stats.prototype.band_pass.call(this, this.min, this.max, false, config);
            b.sum = this.sum;
            b.sum_of_squares = this.sum_of_squares;
            b.sum_of_logs = this.sum_of_logs;
            b.sum_of_square_of_logs = this.sum_of_square_of_logs;
            b.zeroes = this.zeroes;
            return b;
        },
        Σ: function () {
            return this.sum;
        },
        Π: function () {
            return this.zeroes > 0 ? 0 : Math.exp(this.sum_of_logs);
        }
    };
    Stats.prototype.σ = Stats.prototype.stddev;
    Stats.prototype.μ = Stats.prototype.amean;
});

enifed('glimmer-test-helpers/index', ['exports', 'glimmer-test-helpers/lib/helpers', 'glimmer-test-helpers/lib/environment', 'glimmer-test-helpers/lib/abstract-test-case'], function (exports, _glimmerTestHelpersLibHelpers, _glimmerTestHelpersLibEnvironment, _glimmerTestHelpersLibAbstractTestCase) {
  'use strict';

  exports.compile = _glimmerTestHelpersLibHelpers.compile;
  exports.compileLayout = _glimmerTestHelpersLibHelpers.compileLayout;
  exports.equalInnerHTML = _glimmerTestHelpersLibHelpers.equalInnerHTML;
  exports.equalHTML = _glimmerTestHelpersLibHelpers.equalHTML;
  exports.equalTokens = _glimmerTestHelpersLibHelpers.equalTokens;
  exports.generateSnapshot = _glimmerTestHelpersLibHelpers.generateSnapshot;
  exports.equalSnapshots = _glimmerTestHelpersLibHelpers.equalSnapshots;
  exports.normalizeInnerHTML = _glimmerTestHelpersLibHelpers.normalizeInnerHTML;
  exports.isCheckedInputHTML = _glimmerTestHelpersLibHelpers.isCheckedInputHTML;
  exports.getTextContent = _glimmerTestHelpersLibHelpers.getTextContent;
  exports.strip = _glimmerTestHelpersLibHelpers.strip;
  exports.stripTight = _glimmerTestHelpersLibHelpers.stripTight;
  exports.Attrs = _glimmerTestHelpersLibEnvironment.Attrs;
  exports.BasicComponent = _glimmerTestHelpersLibEnvironment.BasicComponent;
  exports.EmberishCurlyComponent = _glimmerTestHelpersLibEnvironment.EmberishCurlyComponent;
  exports.EmberishGlimmerComponent = _glimmerTestHelpersLibEnvironment.EmberishGlimmerComponent;
  exports.TestModifierManager = _glimmerTestHelpersLibEnvironment.TestModifierManager;
  exports.TestEnvironment = _glimmerTestHelpersLibEnvironment.TestEnvironment;
  exports.TestDynamicScope = _glimmerTestHelpersLibEnvironment.TestDynamicScope;
  exports.equalsElement = _glimmerTestHelpersLibEnvironment.equalsElement;
  exports.inspectHooks = _glimmerTestHelpersLibEnvironment.inspectHooks;
  exports.regex = _glimmerTestHelpersLibEnvironment.regex;
  exports.classes = _glimmerTestHelpersLibEnvironment.classes;
  exports.VersionedObject = _glimmerTestHelpersLibAbstractTestCase.VersionedObject;
  exports.testModule = _glimmerTestHelpersLibAbstractTestCase.testModule;
  exports.template = _glimmerTestHelpersLibAbstractTestCase.template;
  exports.RenderingTest = _glimmerTestHelpersLibAbstractTestCase.RenderingTest;
  exports.SimpleRootReference = _glimmerTestHelpersLibAbstractTestCase.SimpleRootReference;
});

enifed('glimmer-test-helpers/lib/abstract-test-case', ['exports', 'glimmer-reference', 'glimmer-test-helpers/lib/environment', 'glimmer-test-helpers/lib/helpers'], function (exports, _glimmerReference, _glimmerTestHelpersLibEnvironment, _glimmerTestHelpersLibHelpers) {
    'use strict';

    exports.skip = skip;
    exports.testModule = testModule;
    exports.template = template;

    function skip(target, name, descriptor) {
        descriptor.value['skip'] = true;
    }

    var VersionedObject = (function () {
        function VersionedObject(value) {
            this.tag = new _glimmerReference.DirtyableTag();
            _glimmerTestHelpersLibHelpers.assign(this, value);
        }

        VersionedObject.prototype.update = function update(value) {
            _glimmerTestHelpersLibHelpers.assign(this, value);
            this.dirty();
        };

        VersionedObject.prototype.set = function set(key, value) {
            this[key] = value;
            this.dirty();
        };

        VersionedObject.prototype.dirty = function dirty() {
            this.tag.dirty();
        };

        return VersionedObject;
    })();

    exports.VersionedObject = VersionedObject;

    var SimpleRootReference = (function () {
        function SimpleRootReference(object) {
            this.object = object;
            this.tag = object.tag;
        }

        SimpleRootReference.prototype.get = function get(key) {
            return new SimplePathReference(this, key);
        };

        SimpleRootReference.prototype.value = function value() {
            return this.object;
        };

        return SimpleRootReference;
    })();

    exports.SimpleRootReference = SimpleRootReference;

    var SimplePathReference = (function () {
        function SimplePathReference(parent, key) {
            this.parent = parent;
            this.key = key;
            this.tag = parent.tag;
        }

        SimplePathReference.prototype.get = function get(key) {
            return new SimplePathReference(this, key);
        };

        SimplePathReference.prototype.value = function value() {
            return this.parent.value()[this.key];
        };

        return SimplePathReference;
    })();

    function isMarker(node) {
        if (node instanceof Comment && node.textContent === '') {
            return true;
        }
        if (node instanceof Text && node.textContent === '') {
            return true;
        }
        return false;
    }

    var RenderingTest = (function () {
        function RenderingTest(env, template, appendTo) {
            if (env === undefined) env = new _glimmerTestHelpersLibEnvironment.TestEnvironment();

            this.env = env;
            this.appendTo = appendTo;
            this.context = null;
            this.result = null;
            this.template = this.env.compile(template);
            this.assert = QUnit.config.current.assert;
        }

        RenderingTest.prototype.teardown = function teardown() {};

        RenderingTest.prototype.render = function render(context) {
            var dynamicScope = new _glimmerTestHelpersLibEnvironment.TestDynamicScope();
            var appendTo = this.appendTo;
            var rootObject = new VersionedObject(context);
            var root = new SimpleRootReference(rootObject);
            this.context = rootObject;
            this.result = this.template.render(root, appendTo, dynamicScope);
            this.element = document.getElementById('qunit-fixture').firstChild;
        };

        RenderingTest.prototype.assertContent = function assertContent(expected, message) {
            var actual = document.getElementById('qunit-fixture').innerHTML;
            QUnit.equal(actual, expected);
        };

        RenderingTest.prototype.takeSnapshot = function takeSnapshot() {
            var snapshot = this.snapshot = [];
            var node = this.element.firstChild;
            while (node) {
                if (!isMarker(node)) {
                    snapshot.push(node);
                }
                node = node.nextSibling;
            }
            return snapshot;
        };

        RenderingTest.prototype.assertStableRerender = function assertStableRerender() {
            this.takeSnapshot();
            this.rerender();
            this.assertInvariants();
        };

        RenderingTest.prototype.rerender = function rerender() {
            this.result.rerender();
        };

        RenderingTest.prototype.assertInvariants = function assertInvariants(oldSnapshot, newSnapshot) {
            oldSnapshot = oldSnapshot || this.snapshot;
            newSnapshot = newSnapshot || this.takeSnapshot();
            this.assert.strictEqual(newSnapshot.length, oldSnapshot.length, 'Same number of nodes');
            for (var i = 0; i < oldSnapshot.length; i++) {
                this.assertSameNode(newSnapshot[i], oldSnapshot[i]);
            }
        };

        RenderingTest.prototype.assertSameNode = function assertSameNode(actual, expected) {
            this.assert.strictEqual(actual, expected, 'DOM node stability');
        };

        RenderingTest.prototype.runTask = function runTask(callback) {
            callback();
            this.result.rerender();
        };

        return RenderingTest;
    })();

    exports.RenderingTest = RenderingTest;

    function testModule(description) {
        return function (TestClass) {
            var context = undefined;
            QUnit.module('[Browser] ' + (description || TestClass.name), {
                teardown: function () {
                    context.teardown();
                }
            });
            var keys = Object.getOwnPropertyNames(TestClass.prototype);
            keys.forEach(function (key) {
                if (key === 'constructor') return;
                var value = Object.getOwnPropertyDescriptor(TestClass.prototype, key).value;
                var isSkipped = value.skip;
                if (typeof value === 'function' && !isSkipped) {
                    QUnit.test(key, function (assert) {
                        var env = new _glimmerTestHelpersLibEnvironment.TestEnvironment();
                        context = new TestClass(env, value['template'], document.getElementById('qunit-fixture'));
                        value.call(context, assert);
                    });
                } else if (isSkipped) {
                    QUnit.skip(key, function () {});
                }
            });
        };
    }

    function template(t) {
        return function template(target, name, descriptor) {
            if (typeof descriptor.value !== 'function') {
                throw new Error("Can't decorator a non-function with the @template decorator");
            }
            descriptor.value['template'] = t;
        };
    }
});

enifed("glimmer-test-helpers/lib/environment", ["exports", "glimmer-runtime", "glimmer-test-helpers/lib/helpers", "glimmer-util", "glimmer-object", "glimmer-reference", "glimmer-object-reference"], function (exports, _glimmerRuntime, _glimmerTestHelpersLibHelpers, _glimmerUtil, _glimmerObject, _glimmerReference, _glimmerObjectReference) {
    "use strict";

    exports.inspectHooks = inspectHooks;
    exports.equalsElement = equalsElement;
    exports.equalsAttr = equalsAttr;
    exports.equals = equals;
    exports.regex = regex;
    exports.classes = classes;

    var ArrayIterator = (function () {
        function ArrayIterator(array, keyFor) {
            this.position = 0;
            this.array = array;
            this.keyFor = keyFor;
        }

        ArrayIterator.prototype.isEmpty = function isEmpty() {
            return this.array.length === 0;
        };

        ArrayIterator.prototype.next = function next() {
            var position = this.position;
            var array = this.array;
            var keyFor = this.keyFor;

            if (position >= array.length) return null;
            var value = array[position];
            var key = keyFor(value, position);
            var memo = position;
            this.position++;
            return { key: key, value: value, memo: memo };
        };

        return ArrayIterator;
    })();

    var ObjectKeysIterator = (function () {
        function ObjectKeysIterator(keys, values, keyFor) {
            this.position = 0;
            this.keys = keys;
            this.values = values;
            this.keyFor = keyFor;
        }

        ObjectKeysIterator.prototype.isEmpty = function isEmpty() {
            return this.keys.length === 0;
        };

        ObjectKeysIterator.prototype.next = function next() {
            var position = this.position;
            var keys = this.keys;
            var values = this.values;
            var keyFor = this.keyFor;

            if (position >= keys.length) return null;
            var value = values[position];
            var memo = keys[position];
            var key = keyFor(value, memo);
            this.position++;
            return { key: key, value: value, memo: memo };
        };

        return ObjectKeysIterator;
    })();

    var EmptyIterator = (function () {
        function EmptyIterator() {}

        EmptyIterator.prototype.isEmpty = function isEmpty() {
            return true;
        };

        EmptyIterator.prototype.next = function next() {
            throw new Error("Cannot call next() on an empty iterator");
        };

        return EmptyIterator;
    })();

    var EMPTY_ITERATOR = new EmptyIterator();

    var Iterable = (function () {
        function Iterable(ref, keyFor) {
            this.tag = ref.tag;
            this.ref = ref;
            this.keyFor = keyFor;
        }

        Iterable.prototype.iterate = function iterate() {
            var ref = this.ref;
            var keyFor = this.keyFor;

            var iterable = ref.value();
            if (Array.isArray(iterable)) {
                return iterable.length > 0 ? new ArrayIterator(iterable, keyFor) : EMPTY_ITERATOR;
            } else if (iterable === undefined || iterable === null) {
                return EMPTY_ITERATOR;
            } else if (iterable.forEach !== undefined) {
                var _ret = (function () {
                    var array = [];
                    iterable.forEach(function (item) {
                        array.push(item);
                    });
                    return {
                        v: array.length > 0 ? new ArrayIterator(array, keyFor) : EMPTY_ITERATOR
                    };
                })();

                if (typeof _ret === "object") return _ret.v;
            } else if (typeof iterable === 'object') {
                var keys = Object.keys(iterable);
                return keys.length > 0 ? new ObjectKeysIterator(keys, keys.map(function (key) {
                    return iterable[key];
                }), keyFor) : EMPTY_ITERATOR;
            } else {
                throw new Error("Don't know how to {{#each " + iterable + "}}");
            }
        };

        Iterable.prototype.valueReferenceFor = function valueReferenceFor(item) {
            return new _glimmerObjectReference.UpdatableReference(item.value);
        };

        Iterable.prototype.updateValueReference = function updateValueReference(reference, item) {
            reference.update(item.value);
        };

        Iterable.prototype.memoReferenceFor = function memoReferenceFor(item) {
            return new _glimmerObjectReference.UpdatableReference(item.memo);
        };

        Iterable.prototype.updateMemoReference = function updateMemoReference(reference, item) {
            reference.update(item.memo);
        };

        return Iterable;
    })();

    var BasicComponent = function BasicComponent(attrs) {
        this.attrs = attrs;
    };

    exports.BasicComponent = BasicComponent;

    var EmberishCurlyComponent = (function (_GlimmerObject) {
        babelHelpers.inherits(EmberishCurlyComponent, _GlimmerObject);

        function EmberishCurlyComponent() {
            _GlimmerObject.apply(this, arguments);
            this.dirtinessTag = new _glimmerReference.DirtyableTag();
            this.tagName = null;
            this.attributeBindings = null;
            this.parentView = null;
        }

        EmberishCurlyComponent.create = function create(args) {
            return _GlimmerObject.create.call(this, args);
        };

        EmberishCurlyComponent.prototype.recompute = function recompute() {
            this.dirtinessTag.dirty();
        };

        EmberishCurlyComponent.prototype.didInitAttrs = function didInitAttrs(options) {};

        EmberishCurlyComponent.prototype.didUpdateAttrs = function didUpdateAttrs(diff) {};

        EmberishCurlyComponent.prototype.didReceiveAttrs = function didReceiveAttrs(diff) {};

        EmberishCurlyComponent.prototype.willInsertElement = function willInsertElement() {};

        EmberishCurlyComponent.prototype.willUpdate = function willUpdate() {};

        EmberishCurlyComponent.prototype.willRender = function willRender() {};

        EmberishCurlyComponent.prototype.didInsertElement = function didInsertElement() {};

        EmberishCurlyComponent.prototype.didUpdate = function didUpdate() {};

        EmberishCurlyComponent.prototype.didRender = function didRender() {};

        return EmberishCurlyComponent;
    })(_glimmerObject.default);

    exports.EmberishCurlyComponent = EmberishCurlyComponent;

    var EmberishGlimmerComponent = (function (_GlimmerObject2) {
        babelHelpers.inherits(EmberishGlimmerComponent, _GlimmerObject2);

        function EmberishGlimmerComponent() {
            _GlimmerObject2.apply(this, arguments);
            this.dirtinessTag = new _glimmerReference.DirtyableTag();
            this.parentView = null;
        }

        EmberishGlimmerComponent.create = function create(args) {
            return _GlimmerObject2.create.call(this, args);
        };

        EmberishGlimmerComponent.prototype.recompute = function recompute() {
            this.dirtinessTag.dirty();
        };

        EmberishGlimmerComponent.prototype.didInitAttrs = function didInitAttrs(options) {};

        EmberishGlimmerComponent.prototype.didUpdateAttrs = function didUpdateAttrs(diff) {};

        EmberishGlimmerComponent.prototype.didReceiveAttrs = function didReceiveAttrs(diff) {};

        EmberishGlimmerComponent.prototype.willInsertElement = function willInsertElement() {};

        EmberishGlimmerComponent.prototype.willUpdate = function willUpdate() {};

        EmberishGlimmerComponent.prototype.willRender = function willRender() {};

        EmberishGlimmerComponent.prototype.didInsertElement = function didInsertElement() {};

        EmberishGlimmerComponent.prototype.didUpdate = function didUpdate() {};

        EmberishGlimmerComponent.prototype.didRender = function didRender() {};

        return EmberishGlimmerComponent;
    })(_glimmerObject.default);

    exports.EmberishGlimmerComponent = EmberishGlimmerComponent;

    var BasicComponentManager = (function () {
        function BasicComponentManager() {}

        BasicComponentManager.prototype.prepareArgs = function prepareArgs(definition, args) {
            return args;
        };

        BasicComponentManager.prototype.create = function create(environment, definition, args) {
            var klass = definition.ComponentClass || BasicComponent;
            return new klass(args.named.value());
        };

        BasicComponentManager.prototype.layoutFor = function layoutFor(definition, component, env) {
            var layout = env.compiledLayouts[definition.name];
            if (layout) {
                return layout;
            }
            return env.compiledLayouts[definition.name] = _glimmerRuntime.compileLayout(new BasicComponentLayoutCompiler(definition.layoutString), env);
        };

        BasicComponentManager.prototype.getSelf = function getSelf(component) {
            return new _glimmerObjectReference.UpdatableReference(component);
        };

        BasicComponentManager.prototype.didCreateElement = function didCreateElement(component, element) {
            component.element = element;
        };

        BasicComponentManager.prototype.didRenderLayout = function didRenderLayout(component, bounds) {
            component.bounds = bounds;
        };

        BasicComponentManager.prototype.didCreate = function didCreate() {};

        BasicComponentManager.prototype.getTag = function getTag() {
            return null;
        };

        BasicComponentManager.prototype.update = function update(component, attrs) {
            component.attrs = attrs.named.value();
        };

        BasicComponentManager.prototype.didUpdateLayout = function didUpdateLayout() {};

        BasicComponentManager.prototype.didUpdate = function didUpdate() {};

        BasicComponentManager.prototype.getDestructor = function getDestructor() {
            return null;
        };

        return BasicComponentManager;
    })();

    var BASIC_COMPONENT_MANAGER = new BasicComponentManager();

    var StaticTaglessComponentManager = (function (_BasicComponentManager) {
        babelHelpers.inherits(StaticTaglessComponentManager, _BasicComponentManager);

        function StaticTaglessComponentManager() {
            _BasicComponentManager.apply(this, arguments);
        }

        StaticTaglessComponentManager.prototype.layoutFor = function layoutFor(definition, component, env) {
            var layout = env.compiledLayouts[definition.name];
            if (layout) {
                return layout;
            }
            return env.compiledLayouts[definition.name] = _glimmerRuntime.compileLayout(new StaticTaglessComponentLayoutCompiler(definition.layoutString), env);
        };

        return StaticTaglessComponentManager;
    })(BasicComponentManager);

    var STATIC_TAGLESS_COMPONENT_MANAGER = new StaticTaglessComponentManager();
    var BaseEmberishGlimmerComponent = EmberishGlimmerComponent.extend();

    var EmberishGlimmerComponentManager = (function () {
        function EmberishGlimmerComponentManager() {}

        EmberishGlimmerComponentManager.prototype.prepareArgs = function prepareArgs(definition, args) {
            return args;
        };

        EmberishGlimmerComponentManager.prototype.create = function create(environment, definition, args, dynamicScope, callerSelf, hasDefaultBlock) {
            var klass = definition.ComponentClass || BaseEmberishGlimmerComponent;
            var attrs = args.named.value();
            var component = klass.create({ attrs: attrs });
            component.didInitAttrs({ attrs: attrs });
            component.didReceiveAttrs({ oldAttrs: null, newAttrs: attrs });
            component.willInsertElement();
            component.willRender();
            return component;
        };

        EmberishGlimmerComponentManager.prototype.layoutFor = function layoutFor(definition, component, env) {
            if (env.compiledLayouts[definition.name]) {
                return env.compiledLayouts[definition.name];
            }
            return env.compiledLayouts[definition.name] = _glimmerRuntime.compileLayout(new EmberishGlimmerComponentLayoutCompiler(definition.layoutString), env);
        };

        EmberishGlimmerComponentManager.prototype.getSelf = function getSelf(component) {
            return new _glimmerObjectReference.UpdatableReference(component);
        };

        EmberishGlimmerComponentManager.prototype.didCreateElement = function didCreateElement(component, element) {
            component.element = element;
        };

        EmberishGlimmerComponentManager.prototype.didRenderLayout = function didRenderLayout(component, bounds) {
            component.bounds = bounds;
        };

        EmberishGlimmerComponentManager.prototype.didCreate = function didCreate(component) {
            component.didInsertElement();
            component.didRender();
        };

        EmberishGlimmerComponentManager.prototype.getTag = function getTag(component) {
            return component.dirtinessTag;
        };

        EmberishGlimmerComponentManager.prototype.update = function update(component, args) {
            var oldAttrs = component.attrs;
            var newAttrs = args.named.value();
            component.set('attrs', newAttrs);
            component.didUpdateAttrs({ oldAttrs: oldAttrs, newAttrs: newAttrs });
            component.didReceiveAttrs({ oldAttrs: oldAttrs, newAttrs: newAttrs });
            component.willUpdate();
            component.willRender();
        };

        EmberishGlimmerComponentManager.prototype.didUpdateLayout = function didUpdateLayout() {};

        EmberishGlimmerComponentManager.prototype.didUpdate = function didUpdate(component) {
            component.didUpdate();
            component.didRender();
        };

        EmberishGlimmerComponentManager.prototype.getDestructor = function getDestructor(component) {
            return component;
        };

        return EmberishGlimmerComponentManager;
    })();

    var ProcessedArgs = (function () {
        function ProcessedArgs(args, positionalParamsDefinition) {
            this.tag = args.tag;
            this.named = args.named;
            this.positional = args.positional;
            this.positionalParamNames = positionalParamsDefinition;
        }

        ProcessedArgs.prototype.value = function value() {
            var named = this.named;
            var positional = this.positional;
            var positionalParamNames = this.positionalParamNames;

            var merged = Object.assign({}, named.value());
            if (positionalParamNames && positionalParamNames.length) {
                for (var i = 0; i < positionalParamNames.length; i++) {
                    var _name = positionalParamNames[i];
                    var reference = positional.at(i);
                    merged[_name] = reference.value();
                }
            }
            return {
                attrs: merged,
                props: merged
            };
        };

        return ProcessedArgs;
    })();

    exports.ProcessedArgs = ProcessedArgs;

    function processArgs(args, positionalParamsDefinition) {
        return new ProcessedArgs(args, positionalParamsDefinition);
    }
    var EMBERISH_GLIMMER_COMPONENT_MANAGER = new EmberishGlimmerComponentManager();
    var BaseEmberishCurlyComponent = EmberishCurlyComponent.extend();

    var EmberishCurlyComponentManager = (function () {
        function EmberishCurlyComponentManager() {}

        EmberishCurlyComponentManager.prototype.prepareArgs = function prepareArgs(definition, args) {
            return args;
        };

        EmberishCurlyComponentManager.prototype.create = function create(environment, definition, args, dynamicScope, callerSelf) {
            var klass = definition.ComponentClass || BaseEmberishCurlyComponent;
            var processedArgs = processArgs(args, klass['positionalParams']);

            var _processedArgs$value = processedArgs.value();

            var attrs = _processedArgs$value.attrs;

            var self = callerSelf.value();
            var merged = _glimmerUtil.assign({}, attrs, { attrs: attrs }, { args: processedArgs }, { targetObject: self });
            var component = klass.create(merged);
            component.didInitAttrs({ attrs: attrs });
            component.didReceiveAttrs({ oldAttrs: null, newAttrs: attrs });
            component.willInsertElement();
            component.willRender();
            return component;
        };

        EmberishCurlyComponentManager.prototype.layoutFor = function layoutFor(definition, component, env) {
            var layout = env.compiledLayouts[definition.name];
            if (layout) {
                return layout;
            }
            var layoutString = definition.layoutString;
            var lateBound = !layoutString;
            if (!layoutString && layoutString !== '') {
                layoutString = component['layout'];
            }
            layout = _glimmerRuntime.compileLayout(new EmberishCurlyComponentLayoutCompiler(layoutString), env);
            return lateBound ? layout : env.compiledLayouts[definition.name] = layout;
        };

        EmberishCurlyComponentManager.prototype.getSelf = function getSelf(component) {
            return new _glimmerObjectReference.UpdatableReference(component);
        };

        EmberishCurlyComponentManager.prototype.didCreateElement = function didCreateElement(component, element, operations) {
            component.element = element;
            var bindings = component.attributeBindings;
            var rootRef = new _glimmerObjectReference.UpdatableReference(component);
            if (bindings) {
                for (var i = 0; i < bindings.length; i++) {
                    var attribute = bindings[i];
                    var reference = rootRef.get(attribute);
                    operations.addDynamicAttribute(element, attribute, reference, false);
                }
            }
        };

        EmberishCurlyComponentManager.prototype.didRenderLayout = function didRenderLayout(component, bounds) {
            component.bounds = bounds;
        };

        EmberishCurlyComponentManager.prototype.didCreate = function didCreate(component) {
            component.didInsertElement();
            component.didRender();
        };

        EmberishCurlyComponentManager.prototype.getTag = function getTag(component) {
            return component.dirtinessTag;
        };

        EmberishCurlyComponentManager.prototype.update = function update(component, args) {
            var oldAttrs = component.attrs;
            var newAttrs = component.args.value().attrs;
            var merged = _glimmerUtil.assign({}, newAttrs, { attrs: newAttrs });
            component.setProperties(merged);
            component.didUpdateAttrs({ oldAttrs: oldAttrs, newAttrs: newAttrs });
            component.didReceiveAttrs({ oldAttrs: oldAttrs, newAttrs: newAttrs });
            component.willUpdate();
            component.willRender();
        };

        EmberishCurlyComponentManager.prototype.didUpdateLayout = function didUpdateLayout() {};

        EmberishCurlyComponentManager.prototype.didUpdate = function didUpdate(component) {
            component.didUpdate();
            component.didRender();
        };

        EmberishCurlyComponentManager.prototype.getDestructor = function getDestructor(component) {
            return component;
        };

        return EmberishCurlyComponentManager;
    })();

    var EMBERISH_CURLY_COMPONENT_MANAGER = new EmberishCurlyComponentManager();
    function emberToBool(value) {
        if (Array.isArray(value)) {
            return value.length > 0;
        } else {
            return !!value;
        }
    }

    var EmberishConditionalReference = (function (_ConditionalReference) {
        babelHelpers.inherits(EmberishConditionalReference, _ConditionalReference);

        function EmberishConditionalReference() {
            _ConditionalReference.apply(this, arguments);
        }

        EmberishConditionalReference.prototype.toBool = function toBool(value) {
            return emberToBool(value);
        };

        return EmberishConditionalReference;
    })(_glimmerRuntime.ConditionalReference);

    var SimplePathReference = (function () {
        function SimplePathReference(parent, property) {
            this.tag = _glimmerReference.VOLATILE_TAG;
            this.parent = parent;
            this.property = property;
        }

        SimplePathReference.prototype.value = function value() {
            return this.parent.value()[this.property];
        };

        SimplePathReference.prototype.get = function get(prop) {
            return new SimplePathReference(this, prop);
        };

        return SimplePathReference;
    })();

    exports.SimplePathReference = SimplePathReference;

    var HelperReference = (function () {
        function HelperReference(helper, args) {
            this.tag = _glimmerReference.VOLATILE_TAG;
            this.helper = helper;
            this.args = args;
        }

        HelperReference.prototype.value = function value() {
            var helper = this.helper;
            var _args = this.args;
            var positional = _args.positional;
            var named = _args.named;

            return helper(positional.value(), named.value());
        };

        HelperReference.prototype.get = function get(prop) {
            return new SimplePathReference(this, prop);
        };

        return HelperReference;
    })();

    var InertModifierManager = (function () {
        function InertModifierManager() {}

        InertModifierManager.prototype.create = function create() {};

        InertModifierManager.prototype.install = function install(modifier) {};

        InertModifierManager.prototype.update = function update(modifier) {};

        InertModifierManager.prototype.getDestructor = function getDestructor(modifier) {
            return null;
        };

        return InertModifierManager;
    })();

    var TestModifier = function TestModifier(element, args, dom) {
        this.element = element;
        this.args = args;
        this.dom = dom;
    };

    exports.TestModifier = TestModifier;

    var TestModifierManager = (function () {
        function TestModifierManager() {
            this.installedElements = [];
            this.updatedElements = [];
            this.destroyedModifiers = [];
        }

        TestModifierManager.prototype.create = function create(element, args, dynamicScope, dom) {
            return new TestModifier(element, args, dom);
        };

        TestModifierManager.prototype.install = function install(_ref) {
            var element = _ref.element;
            var args = _ref.args;
            var dom = _ref.dom;

            this.installedElements.push(element);
            var param = args.positional.at(0).value();
            dom.setAttribute(element, 'data-modifier', "installed - " + param);
            return;
        };

        TestModifierManager.prototype.update = function update(_ref2) {
            var element = _ref2.element;
            var args = _ref2.args;
            var dom = _ref2.dom;

            this.updatedElements.push(element);
            var param = args.positional.at(0).value();
            dom.setAttribute(element, 'data-modifier', "updated - " + param);
            return;
        };

        TestModifierManager.prototype.getDestructor = function getDestructor(modifier) {
            var _this = this;

            return {
                destroy: function () {
                    _this.destroyedModifiers.push(modifier);
                    var element = modifier.element;
                    var dom = modifier.dom;

                    dom.removeAttribute(element, 'data-modifier');
                }
            };
        };

        return TestModifierManager;
    })();

    exports.TestModifierManager = TestModifierManager;

    var TestEnvironment = (function (_Environment) {
        babelHelpers.inherits(TestEnvironment, _Environment);

        function TestEnvironment() {
            var options = arguments.length <= 0 || arguments[0] === undefined ? {
                document: document,
                appendOperations: new _glimmerRuntime.DOMTreeConstruction(document)
            } : arguments[0];

            _Environment.call(this, { appendOperations: options.appendOperations, updateOperations: new _glimmerRuntime.DOMChanges(options.document) });
            this.helpers = _glimmerUtil.dict();
            this.modifiers = _glimmerUtil.dict();
            this.partials = _glimmerUtil.dict();
            this.components = _glimmerUtil.dict();
            this.compiledLayouts = _glimmerUtil.dict();
            this.uselessAnchor = options.document.createElement('a');
            this.registerHelper("if", function (_ref3) {
                var cond = _ref3[0];
                var yes = _ref3[1];
                var no = _ref3[2];
                return cond ? yes : no;
            });
            this.registerHelper("unless", function (_ref4) {
                var cond = _ref4[0];
                var yes = _ref4[1];
                var no = _ref4[2];
                return cond ? no : yes;
            });
            this.registerInternalHelper("-get-dynamic-var", _glimmerRuntime.getDynamicVar);
            this.registerModifier("action", new InertModifierManager());
            this.registerInternalHelper("component", function (vm, args, symbolTable) {
                return new DynamicComponentReference(args.positional.at(0), vm.env, symbolTable);
            });
            this.registerInternalHelper("hash", function (vm, args) {
                return args.named;
            });
        }

        TestEnvironment.prototype.protocolForURL = function protocolForURL(url) {
            this.uselessAnchor.href = url;
            return this.uselessAnchor.protocol;
        };

        TestEnvironment.prototype.registerHelper = function registerHelper(name, helper) {
            this.helpers[name] = function (vm, args) {
                return new HelperReference(helper, args);
            };
        };

        TestEnvironment.prototype.registerInternalHelper = function registerInternalHelper(name, helper) {
            this.helpers[name] = helper;
        };

        TestEnvironment.prototype.registerModifier = function registerModifier(name, modifier) {
            this.modifiers[name] = modifier;
        };

        TestEnvironment.prototype.registerPartial = function registerPartial(name, source) {
            this.partials[name] = new _glimmerRuntime.PartialDefinition(name, _glimmerTestHelpersLibHelpers.compile(source, { env: this }));
        };

        TestEnvironment.prototype.registerComponent = function registerComponent(name, definition) {
            this.components[name] = definition;
            return definition;
        };

        TestEnvironment.prototype.registerBasicComponent = function registerBasicComponent(name, Component, layout) {
            var definition = new BasicComponentDefinition(name, BASIC_COMPONENT_MANAGER, Component, layout);
            return this.registerComponent(name, definition);
        };

        TestEnvironment.prototype.registerStaticTaglessComponent = function registerStaticTaglessComponent(name, Component, layout) {
            var definition = new StaticTaglessComponentDefinition(name, STATIC_TAGLESS_COMPONENT_MANAGER, Component, layout);
            return this.registerComponent(name, definition);
        };

        TestEnvironment.prototype.registerEmberishCurlyComponent = function registerEmberishCurlyComponent(name, Component, layout) {
            var definition = new EmberishCurlyComponentDefinition(name, EMBERISH_CURLY_COMPONENT_MANAGER, Component, layout);
            return this.registerComponent(name, definition);
        };

        TestEnvironment.prototype.registerEmberishGlimmerComponent = function registerEmberishGlimmerComponent(name, Component, layout) {
            var definition = new EmberishGlimmerComponentDefinition(name, EMBERISH_GLIMMER_COMPONENT_MANAGER, Component, layout);
            return this.registerComponent(name, definition);
        };

        TestEnvironment.prototype.toConditionalReference = function toConditionalReference(reference) {
            if (_glimmerReference.isConst(reference)) {
                return new _glimmerRuntime.ValueReference(emberToBool(reference.value()));
            }
            return new EmberishConditionalReference(reference);
        };

        TestEnvironment.prototype.refineStatement = function refineStatement(statement, symbolTable) {
            var appendType = statement.appendType;
            var isSimple = statement.isSimple;
            var isBlock = statement.isBlock;
            var isInline = statement.isInline;
            var key = statement.key;
            var args = statement.args;
            var path = statement.path;
            var templates = statement.templates;

            if (isSimple && isBlock) {
                switch (key) {
                    case 'identity':
                        return new IdentitySyntax({ args: args, templates: templates });
                    case 'render-inverse':
                        return new RenderInverseIdentitySyntax({ args: args, templates: templates });
                    case '-with-dynamic-vars':
                        return new _glimmerRuntime.WithDynamicVarsSyntax({ args: args, templates: templates });
                    case '-in-element':
                        return new _glimmerRuntime.InElementSyntax({ args: args, templates: templates });
                }
            }
            if (isSimple && (isInline || isBlock)) {
                if (key === 'component') {
                    return new DynamicComponentSyntax({ args: args, templates: templates, symbolTable: symbolTable });
                }
                var component = this.getComponentDefinition(path, symbolTable);
                if (component) {
                    return new CurlyComponentSyntax({ args: args, definition: component, templates: templates, symbolTable: symbolTable });
                }
            }
            if (!isSimple && appendType === 'unknown') {
                return statement.original.deopt();
            }
            return _Environment.prototype.refineStatement.call(this, statement, symbolTable);
        };

        TestEnvironment.prototype.hasHelper = function hasHelper(helperName) {
            return helperName.length === 1 && helperName[0] in this.helpers;
        };

        TestEnvironment.prototype.lookupHelper = function lookupHelper(helperParts) {
            var helperName = helperParts[0];
            var helper = this.helpers[helperName];
            if (!helper) throw new Error("Helper for " + helperParts.join('.') + " not found.");
            return helper;
        };

        TestEnvironment.prototype.hasPartial = function hasPartial(partialName) {
            return partialName in this.partials;
        };

        TestEnvironment.prototype.lookupPartial = function lookupPartial(partialName) {
            var partial = this.partials[partialName];
            return partial;
        };

        TestEnvironment.prototype.hasComponentDefinition = function hasComponentDefinition(name) {
            return !!this.components[name[0]];
        };

        TestEnvironment.prototype.getComponentDefinition = function getComponentDefinition(name, blockMeta) {
            return this.components[name[0]];
        };

        TestEnvironment.prototype.hasModifier = function hasModifier(modifierName) {
            return modifierName.length === 1 && modifierName[0] in this.modifiers;
        };

        TestEnvironment.prototype.lookupModifier = function lookupModifier(modifierName) {
            var name = modifierName[0];

            var modifier = this.modifiers[name];
            if (!modifier) throw new Error("Modifier for " + modifierName.join('.') + " not found.");
            return modifier;
        };

        TestEnvironment.prototype.compile = function compile(template) {
            return _glimmerTestHelpersLibHelpers.compile(template, { env: this });
        };

        TestEnvironment.prototype.compileLayout = function compileLayout(template) {
            return _glimmerTestHelpersLibHelpers.compileLayout(template, { env: this });
        };

        TestEnvironment.prototype.iterableFor = function iterableFor(ref, args) {
            var keyPath = args.named.get("key").value();
            var keyFor = undefined;
            if (!keyPath) {
                throw new Error('Must specify a key for #each');
            }
            switch (keyPath) {
                case '@index':
                    keyFor = function (_, index) {
                        return String(index);
                    };
                    break;
                case '@primitive':
                    keyFor = function (item) {
                        return String(item);
                    };
                    break;
                default:
                    keyFor = function (item) {
                        return item[keyPath];
                    };
                    break;
            }
            return new Iterable(ref, keyFor);
        };

        return TestEnvironment;
    })(_glimmerRuntime.Environment);

    exports.TestEnvironment = TestEnvironment;

    var TestDynamicScope = (function () {
        function TestDynamicScope() {}

        TestDynamicScope.prototype.get = function get(key) {
            return this[key];
        };

        TestDynamicScope.prototype.set = function set(key, reference) {
            return this[key] = reference;
        };

        TestDynamicScope.prototype.child = function child() {
            return new TestDynamicScope();
        };

        return TestDynamicScope;
    })();

    exports.TestDynamicScope = TestDynamicScope;

    var CurlyComponentSyntax = (function (_StatementSyntax) {
        babelHelpers.inherits(CurlyComponentSyntax, _StatementSyntax);

        function CurlyComponentSyntax(_ref5) {
            var args = _ref5.args;
            var definition = _ref5.definition;
            var templates = _ref5.templates;
            var symbolTable = _ref5.symbolTable;

            _StatementSyntax.call(this);
            this.type = "curly-component";
            this.shadow = null;
            this.args = args;
            this.definition = definition;
            this.templates = templates || _glimmerRuntime.Templates.empty();
            this.symbolTable = symbolTable;
        }

        CurlyComponentSyntax.prototype.compile = function compile(b, env) {
            b.component.static(this.definition, this.args, this.templates, this.symbolTable, this.shadow);
        };

        return CurlyComponentSyntax;
    })(_glimmerRuntime.StatementSyntax);

    var DynamicComponentReference = (function () {
        function DynamicComponentReference(nameRef, env, symbolTable) {
            this.nameRef = nameRef;
            this.env = env;
            this.symbolTable = symbolTable;
            this.tag = nameRef.tag;
        }

        DynamicComponentReference.prototype.value = function value() {
            var env = this.env;
            var nameRef = this.nameRef;

            var name = nameRef.value();
            if (typeof name === 'string') {
                return env.getComponentDefinition([name], this.symbolTable);
            } else {
                return null;
            }
        };

        DynamicComponentReference.prototype.get = function get() {
            return null;
        };

        return DynamicComponentReference;
    })();

    function dynamicComponentFor(vm, symbolTable) {
        var args = vm.getArgs();
        var nameRef = args.positional.at(0);
        var env = vm.env;
        return new DynamicComponentReference(nameRef, env, symbolTable);
    }
    ;

    var DynamicComponentSyntax = (function (_StatementSyntax2) {
        babelHelpers.inherits(DynamicComponentSyntax, _StatementSyntax2);

        function DynamicComponentSyntax(_ref6) {
            var args = _ref6.args;
            var templates = _ref6.templates;
            var symbolTable = _ref6.symbolTable;

            _StatementSyntax2.call(this);
            this.type = "dynamic-component";
            this.shadow = null;
            this.definitionArgs = _glimmerRuntime.ArgsSyntax.fromPositionalArgs(args.positional.slice(0, 1));
            this.definition = dynamicComponentFor;
            this.args = _glimmerRuntime.ArgsSyntax.build(args.positional.slice(1), args.named);
            this.templates = templates || _glimmerRuntime.Templates.empty();
            this.symbolTable = symbolTable;
        }

        DynamicComponentSyntax.prototype.compile = function compile(b, env) {
            b.component.dynamic(this.definitionArgs, this.definition, this.args, this.templates, this.symbolTable, this.shadow);
        };

        return DynamicComponentSyntax;
    })(_glimmerRuntime.StatementSyntax);

    var GenericComponentDefinition = (function (_ComponentDefinition) {
        babelHelpers.inherits(GenericComponentDefinition, _ComponentDefinition);

        function GenericComponentDefinition(name, manager, ComponentClass, layout) {
            _ComponentDefinition.call(this, name, manager, ComponentClass);
            this.layoutString = layout;
        }

        return GenericComponentDefinition;
    })(_glimmerRuntime.ComponentDefinition);

    exports.GenericComponentDefinition = GenericComponentDefinition;

    var BasicComponentDefinition = (function (_GenericComponentDefinition) {
        babelHelpers.inherits(BasicComponentDefinition, _GenericComponentDefinition);

        function BasicComponentDefinition() {
            _GenericComponentDefinition.apply(this, arguments);
        }

        return BasicComponentDefinition;
    })(GenericComponentDefinition);

    exports.BasicComponentDefinition = BasicComponentDefinition;

    var StaticTaglessComponentDefinition = (function (_GenericComponentDefinition2) {
        babelHelpers.inherits(StaticTaglessComponentDefinition, _GenericComponentDefinition2);

        function StaticTaglessComponentDefinition() {
            _GenericComponentDefinition2.apply(this, arguments);
        }

        return StaticTaglessComponentDefinition;
    })(GenericComponentDefinition);

    var EmberishCurlyComponentDefinition = (function (_GenericComponentDefinition3) {
        babelHelpers.inherits(EmberishCurlyComponentDefinition, _GenericComponentDefinition3);

        function EmberishCurlyComponentDefinition() {
            _GenericComponentDefinition3.apply(this, arguments);
        }

        return EmberishCurlyComponentDefinition;
    })(GenericComponentDefinition);

    exports.EmberishCurlyComponentDefinition = EmberishCurlyComponentDefinition;

    var EmberishGlimmerComponentDefinition = (function (_GenericComponentDefinition4) {
        babelHelpers.inherits(EmberishGlimmerComponentDefinition, _GenericComponentDefinition4);

        function EmberishGlimmerComponentDefinition() {
            _GenericComponentDefinition4.apply(this, arguments);
        }

        return EmberishGlimmerComponentDefinition;
    })(GenericComponentDefinition);

    exports.EmberishGlimmerComponentDefinition = EmberishGlimmerComponentDefinition;

    var GenericComponentLayoutCompiler = (function () {
        function GenericComponentLayoutCompiler(layoutString) {
            this.layoutString = layoutString;
        }

        GenericComponentLayoutCompiler.prototype.compileLayout = function compileLayout(env) {
            return _glimmerTestHelpersLibHelpers.compileLayout(this.layoutString, { env: env });
        };

        return GenericComponentLayoutCompiler;
    })();

    var BasicComponentLayoutCompiler = (function (_GenericComponentLayoutCompiler) {
        babelHelpers.inherits(BasicComponentLayoutCompiler, _GenericComponentLayoutCompiler);

        function BasicComponentLayoutCompiler() {
            _GenericComponentLayoutCompiler.apply(this, arguments);
        }

        BasicComponentLayoutCompiler.prototype.compile = function compile(builder) {
            builder.fromLayout(this.compileLayout(builder.env));
        };

        return BasicComponentLayoutCompiler;
    })(GenericComponentLayoutCompiler);

    var StaticTaglessComponentLayoutCompiler = (function (_GenericComponentLayoutCompiler2) {
        babelHelpers.inherits(StaticTaglessComponentLayoutCompiler, _GenericComponentLayoutCompiler2);

        function StaticTaglessComponentLayoutCompiler() {
            _GenericComponentLayoutCompiler2.apply(this, arguments);
        }

        StaticTaglessComponentLayoutCompiler.prototype.compile = function compile(builder) {
            builder.wrapLayout(this.compileLayout(builder.env));
        };

        return StaticTaglessComponentLayoutCompiler;
    })(GenericComponentLayoutCompiler);

    function EmberTagName(vm) {
        var self = vm.getSelf().value();
        var tagName = self['tagName'];
        tagName = tagName === '' ? null : self['tagName'] || 'div';
        return new _glimmerRuntime.ValueReference(tagName);
    }
    function EmberID(vm) {
        var self = vm.getSelf().value();
        return new _glimmerRuntime.ValueReference("ember" + self._guid);
    }

    var EmberishCurlyComponentLayoutCompiler = (function (_GenericComponentLayoutCompiler3) {
        babelHelpers.inherits(EmberishCurlyComponentLayoutCompiler, _GenericComponentLayoutCompiler3);

        function EmberishCurlyComponentLayoutCompiler() {
            _GenericComponentLayoutCompiler3.apply(this, arguments);
        }

        EmberishCurlyComponentLayoutCompiler.prototype.compile = function compile(builder) {
            builder.wrapLayout(this.compileLayout(builder.env));
            builder.tag.dynamic(EmberTagName);
            builder.attrs.static('class', 'ember-view');
            builder.attrs.dynamic('id', EmberID);
        };

        return EmberishCurlyComponentLayoutCompiler;
    })(GenericComponentLayoutCompiler);

    var EmberishGlimmerComponentLayoutCompiler = (function (_GenericComponentLayoutCompiler4) {
        babelHelpers.inherits(EmberishGlimmerComponentLayoutCompiler, _GenericComponentLayoutCompiler4);

        function EmberishGlimmerComponentLayoutCompiler() {
            _GenericComponentLayoutCompiler4.apply(this, arguments);
        }

        EmberishGlimmerComponentLayoutCompiler.prototype.compile = function compile(builder) {
            builder.fromLayout(this.compileLayout(builder.env));
            builder.attrs.static('class', 'ember-view');
            builder.attrs.dynamic('id', EmberID);
        };

        return EmberishGlimmerComponentLayoutCompiler;
    })(GenericComponentLayoutCompiler);

    function inspectHooks(ComponentClass) {
        return ComponentClass.extend({
            init: function () {
                this._super.apply(this, arguments);
                this.hooks = {
                    didInitAttrs: 0,
                    didUpdateAttrs: 0,
                    didReceiveAttrs: 0,
                    willInsertElement: 0,
                    willUpdate: 0,
                    willRender: 0,
                    didInsertElement: 0,
                    didUpdate: 0,
                    didRender: 0
                };
            },
            didInitAttrs: function () {
                this._super.apply(this, arguments);
                this.hooks['didInitAttrs']++;
            },
            didUpdateAttrs: function () {
                this._super.apply(this, arguments);
                this.hooks['didUpdateAttrs']++;
            },
            didReceiveAttrs: function () {
                this._super.apply(this, arguments);
                this.hooks['didReceiveAttrs']++;
            },
            willInsertElement: function () {
                this._super.apply(this, arguments);
                this.hooks['willInsertElement']++;
            },
            willUpdate: function () {
                this._super.apply(this, arguments);
                this.hooks['willUpdate']++;
            },
            willRender: function () {
                this._super.apply(this, arguments);
                this.hooks['willRender']++;
            },
            didInsertElement: function () {
                this._super.apply(this, arguments);
                this.hooks['didInsertElement']++;
            },
            didUpdate: function () {
                this._super.apply(this, arguments);
                this.hooks['didUpdate']++;
            },
            didRender: function () {
                this._super.apply(this, arguments);
                this.hooks['didRender']++;
            }
        });
    }

    var IdentitySyntax = (function (_StatementSyntax3) {
        babelHelpers.inherits(IdentitySyntax, _StatementSyntax3);

        function IdentitySyntax(_ref7) {
            var args = _ref7.args;
            var templates = _ref7.templates;

            _StatementSyntax3.call(this);
            this.type = "identity";
            this.args = args;
            this.templates = templates;
        }

        IdentitySyntax.prototype.compile = function compile(dsl) {
            dsl.evaluate('default', this.templates.default);
        };

        return IdentitySyntax;
    })(_glimmerRuntime.StatementSyntax);

    var RenderInverseIdentitySyntax = (function (_StatementSyntax4) {
        babelHelpers.inherits(RenderInverseIdentitySyntax, _StatementSyntax4);

        function RenderInverseIdentitySyntax(_ref8) {
            var args = _ref8.args;
            var templates = _ref8.templates;

            _StatementSyntax4.call(this);
            this.type = "render-inverse-identity";
            this.args = args;
            this.templates = templates;
        }

        RenderInverseIdentitySyntax.prototype.compile = function compile(dsl) {
            dsl.evaluate('inverse', this.templates.inverse);
        };

        return RenderInverseIdentitySyntax;
    })(_glimmerRuntime.StatementSyntax);

    function equalsElement(element, tagName, attributes, content) {
        QUnit.push(element.tagName === tagName.toUpperCase(), element.tagName.toLowerCase(), tagName, "expect tagName to be " + tagName);
        var expectedAttrs = _glimmerUtil.dict();
        var expectedCount = 0;
        for (var prop in attributes) {
            expectedCount++;
            var expected = attributes[prop];
            var matcher = typeof expected === 'object' && MATCHER in expected ? expected : equalsAttr(expected);
            expectedAttrs[prop] = matcher;
            QUnit.push(expectedAttrs[prop].match(element.getAttribute(prop)), matcher.fail(element.getAttribute(prop)), matcher.fail(element.getAttribute(prop)), "Expected element's " + prop + " attribute " + matcher.expected());
        }
        var actualAttributes = {};
        for (var i = 0, l = element.attributes.length; i < l; i++) {
            actualAttributes[element.attributes[i].name] = element.attributes[i].value;
        }
        if (!(element instanceof HTMLElement)) {
            QUnit.push(element instanceof HTMLElement, null, null, "Element must be an HTML Element, not an SVG Element");
        } else {
            QUnit.push(element.attributes.length === expectedCount, element.attributes.length, expectedCount, "Expected " + expectedCount + " attributes; got " + element.outerHTML);
            if (content !== null) {
                QUnit.push(element.innerHTML === content, element.innerHTML, content, "The element had '" + content + "' as its content");
            }
        }
    }

    var MATCHER = "3d4ef194-13be-4ccf-8dc7-862eea02c93e";
    exports.MATCHER = MATCHER;

    function equalsAttr(expected) {
        return {
            "3d4ef194-13be-4ccf-8dc7-862eea02c93e": true,
            match: function (actual) {
                return expected === actual;
            },
            expected: function () {
                return "to equal " + expected;
            },
            fail: function (actual) {
                return actual + " did not equal " + expected;
            }
        };
    }

    function equals(expected) {
        return {
            "3d4ef194-13be-4ccf-8dc7-862eea02c93e": true,
            match: function (actual) {
                return expected === actual;
            },
            expected: function () {
                return "to equal " + expected;
            },
            fail: function (actual) {
                return actual + " did not equal " + expected;
            }
        };
    }

    function regex(r) {
        return {
            "3d4ef194-13be-4ccf-8dc7-862eea02c93e": true,
            match: function (v) {
                return r.test(v);
            },
            expected: function () {
                return "to match " + r;
            },
            fail: function (actual) {
                return actual + " did not match " + r;
            }
        };
    }

    function classes(expected) {
        return {
            "3d4ef194-13be-4ccf-8dc7-862eea02c93e": true,
            match: function (actual) {
                return actual && expected.split(' ').sort().join(' ') === actual.split(' ').sort().join(' ');
            },
            expected: function () {
                return "to include '" + expected + "'";
            },
            fail: function (actual) {
                return "'" + actual + "'' did not match '" + expected + "'";
            }
        };
    }
});

// Partials

// Syntax Classes

// Concrete Syntax

// References

enifed("glimmer-test-helpers/lib/helpers", ["exports", "simple-html-tokenizer", "glimmer-runtime", "glimmer-compiler"], function (exports, _simpleHtmlTokenizer, _glimmerRuntime, _glimmerCompiler) {
    "use strict";

    exports.compile = compile;
    exports.compileLayout = compileLayout;
    exports.equalInnerHTML = equalInnerHTML;
    exports.equalHTML = equalHTML;
    exports.equalTokens = equalTokens;
    exports.generateSnapshot = generateSnapshot;
    exports.equalSnapshots = equalSnapshots;
    exports.normalizeInnerHTML = normalizeInnerHTML;
    exports.getTextContent = getTextContent;
    exports.strip = strip;
    exports.stripTight = stripTight;

    // For Phantom
    function toObject(val) {
        if (val === null || val === undefined) {
            throw new TypeError('Object.assign cannot be called with null or undefined');
        }
        return Object(val);
    }
    if (typeof Object.assign !== 'function') {
        Object.assign = function (target, source) {
            var from = undefined;
            var to = toObject(target);
            var symbols = undefined;
            for (var s = 1; s < arguments.length; s++) {
                from = Object(arguments[s]);
                for (var key in from) {
                    if (Object.prototype.hasOwnProperty.call(from, key)) {
                        to[key] = from[key];
                    }
                }
                if (Object.getOwnPropertySymbols) {
                    symbols = Object.getOwnPropertySymbols(from);
                    for (var i = 0; i < symbols.length; i++) {
                        if (Object.prototype.propertyIsEnumerable.call(from, symbols[i])) {
                            to[symbols[i]] = from[symbols[i]];
                        }
                    }
                }
            }
            return to;
        };
    }
    var assign = Object.assign;
    exports.assign = assign;
    function isMarker(node) {
        var TextNode = window.Text;
        var Comment = window.Comment;
        if (node instanceof Comment && node.textContent === '') {
            return true;
        }
        if (node instanceof TextNode && node.textContent === '') {
            return true;
        }
        return false;
    }

    function compile(string, options) {
        var js = _glimmerCompiler.precompile(string, options);
        var factory = _glimmerRuntime.templateFactory(JSON.parse(js));
        return factory.create(options.env);
    }

    function compileLayout(string, options) {
        return compile(string, options).asLayout();
    }

    function equalInnerHTML(fragment, html, msg) {
        var actualHTML = normalizeInnerHTML(fragment.innerHTML);
        QUnit.push(actualHTML === html, actualHTML, html, msg);
    }

    function equalHTML(node, html) {
        var fragment = undefined;
        if (!node.nodeType && node.length) {
            fragment = document.createDocumentFragment();
            while (node[0]) {
                fragment.appendChild(node[0]);
            }
        } else {
            fragment = node;
        }
        var div = document.createElement("div");
        div.appendChild(fragment.cloneNode(true));
        equalInnerHTML(div, html);
    }

    function generateTokens(divOrHTML) {
        var div = undefined;
        if (typeof divOrHTML === 'string') {
            div = document.createElement("div");
            div.innerHTML = divOrHTML;
        } else {
            div = divOrHTML;
        }
        return { tokens: _simpleHtmlTokenizer.tokenize(div.innerHTML), html: div.innerHTML };
    }

    function equalTokens(fragment, html) {
        var message = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];

        if (fragment.fragment) {
            fragment = fragment.fragment;
        }
        if (html.fragment) {
            html = html.fragment;
        }
        var fragTokens = generateTokens(fragment);
        var htmlTokens = generateTokens(html);
        function normalizeTokens(token) {
            if (token.type === 'StartTag') {
                token.attributes = token.attributes.sort(function (a, b) {
                    if (a[0] > b[0]) {
                        return 1;
                    }
                    if (a[0] < b[0]) {
                        return -1;
                    }
                    return 0;
                });
            }
        }
        fragTokens.tokens.forEach(normalizeTokens);
        htmlTokens.tokens.forEach(normalizeTokens);
        // let msg = "Expected: " + htmlTokens.html + "; Actual: " + fragTokens.html;
        // if (message) { msg += " (" + message + ")"; }
        var equiv = QUnit.equiv(fragTokens.tokens, htmlTokens.tokens);
        if (equiv && fragTokens.html !== htmlTokens.html) {
            deepEqual(fragTokens.tokens, htmlTokens.tokens, message);
        } else {
            QUnit.push(QUnit.equiv(fragTokens.tokens, htmlTokens.tokens), fragTokens.html, htmlTokens.html, message);
        }
        // deepEqual(fragTokens.tokens, htmlTokens.tokens, msg);
    }

    function generateSnapshot(element) {
        var snapshot = [];
        var node = element.firstChild;
        while (node) {
            if (!isMarker(node)) {
                snapshot.push(node);
            }
            node = node.nextSibling;
        }
        return snapshot;
    }

    function equalSnapshots(a, b) {
        strictEqual(a.length, b.length, 'Same number of nodes');
        for (var i = 0; i < b.length; i++) {
            strictEqual(a[i], b[i], 'Nodes are the same');
        }
    }

    // detect side-effects of cloning svg elements in IE9-11
    var ieSVGInnerHTML = (function () {
        if (typeof document === 'undefined' || !document.createElementNS) {
            return false;
        }
        var div = document.createElement('div');
        var node = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
        div.appendChild(node);
        var clone = div.cloneNode(true);
        return clone.innerHTML === '<svg xmlns="http://www.w3.org/2000/svg" />';
    })();

    function normalizeInnerHTML(actualHTML) {
        if (ieSVGInnerHTML) {
            // Replace `<svg xmlns="http://www.w3.org/2000/svg" height="50%" />` with `<svg height="50%"></svg>`, etc.
            // drop namespace attribute
            actualHTML = actualHTML.replace(/ xmlns="[^"]+"/, '');
            // replace self-closing elements
            actualHTML = actualHTML.replace(/<([^ >]+) [^\/>]*\/>/gi, function (tag, tagName) {
                return tag.slice(0, tag.length - 3) + '></' + tagName + '>';
            });
        }
        return actualHTML;
    }

    var isCheckedInputHTML = undefined;
    if (typeof document === 'undefined') {
        exports.isCheckedInputHTML = isCheckedInputHTML = function (element) {};
    } else {
        (function () {
            // detect weird IE8 checked element string
            var checkedInput = document.createElement('input');
            checkedInput.setAttribute('checked', 'checked');
            var checkedInputString = checkedInput.outerHTML;
            exports.isCheckedInputHTML = isCheckedInputHTML = function (element) {
                equal(element.outerHTML, checkedInputString);
            };
        })();
    }
    exports.isCheckedInputHTML = isCheckedInputHTML;

    // check which property has the node's text content
    var textProperty = typeof document === 'object' && document.createElement('div').textContent === undefined ? 'innerText' : 'textContent';

    function getTextContent(el) {
        // textNode
        if (el.nodeType === 3) {
            return el.nodeValue;
        } else {
            return el[textProperty];
        }
    }

    function strip(strings) {
        return strings[0].split('\n').map(function (s) {
            return s.trim();
        }).join(' ');
    }

    function stripTight(strings) {
        return strings[0].split('\n').map(function (s) {
            return s.trim();
        }).join('');
    }
});
//# sourceMappingURL=glimmer-benchmarks.amd.map